' John Conway's Game of Life for MMBasic DOS
' Bill McKinley
' 
' Notes for the future
' 1. ClrScr and DrawM could be combined into one sub with a parameter passed
'    to it to tell it whether to write all " " or to write the matrix contents.
' 2. One another platform maybe SETTICK interrupt rather than PAUSE for timing.
' 3. Calculating the state of the next generation cell could be done in a
'    FOR - NEXT loop ignoring the current cell but I think the simple way is
'    quicker.
' 4. Cells outside the range are supposed to be dead the matrix currently
'    has one cell all round not shown and not altered, assumes the outside 
'    cells are initialised to zero when the matrix is DIMed.
'    It still does seem to affect other cells on the edge of the screen
'    because a glider does not disappear of the edge of the screen.
'    Maybe two cells all round with the second to outside one updated
'    but not displayed?
' 5. An amimated version where dying cells fade out and new cells fade in
'    for the Maximite? HMMM maybe not.
' 6. Editor for startup screens similar to CMM sprite files?
'
' NOTE: If you make the matrix too small the Glider demo will cause an error
' and the matrix will not overwrite some of the text on the screen
' Make it too big and it will not fit on the screen
' A ratio of 2 of x to 1 of y gives a better looking display
MatX = 80   ' Matrix horizontal size
MatY = 40   ' Matrix vertical size
RandL = 0.3 ' For the randomizing of life less than =< RandL means there is life
            ' Make it high and you get a lot of cells but they die quickly
PT = 300    ' Pause time in mS between display updates
A$ = "O"    ' The character to display a live cell - X or O are OK 

' Zero based matrix has one more cell all round than is displayed
dim CurM(MatX+1, MatY+1) ' The current matrix of life
dim MewM(MatX+1, MatY+1) ' The next geration of life

ChkScr      ' Check the window size and print intro
SetupTimer  ' Randomize by waiting for a keypress
ClrScr      ' Clear the screen (matrix area)
InitM       ' Initialize the random matrix (or demo)
do          ' Program loop
DrawM       ' Draw the matrix on the screen
NextGen     ' Calculate the next generation
UpdateM     ' Update the matrix with the new generation
pause PT
loop until inkey$ <> "" ' loop forever or until a keypres

sub ChkScr ' Check the window size
w2=0
do
if MatX > mm.hres or MatY > mm.vres then 'Check the size of the DOS window
  w=0
  if w2=0 then 'Not printed the window size message yet
  cursor 3,5:print "Please increase the"
  cursor 3,6:print "Window size to see"
  cursor 3,7:print "  full matrix"
  w2=1
  endif
else
  w=1
end if
loop until w=1
ClrScr
cursor 3,5:print "CONWAY'S GAME OF LIFE"
cursor 3,6:print space$(20)
cursor 3,7:print "Press a key to start"
cursor 3,8:print "and another to stop"
cursor 3,9:print "G for a glider demo"
end sub

sub SetupTimer ' Wait for a key press to start timer
Do
  B$ = inkey$
loop until B$ <> ""
randomize timer
end sub

sub ClrScr 'Clear the screen area
cursor 0,0
for y=1 to MatY
  for x=1 to MatX
    print " ";
  next x
  print
next y
end sub

sub InitM ' Initialise the matrix of life
if B$ = "g" or b$ = "G" then  'Set thr glider and target
  CurM(4,7) =1:CurM(5,7) =1:CurM(6,7)=1:CurM(6,6)=1: CurM(5,5)=1
  CurM(15,15) =1:CurM(15,16) =1:CurM(16,15)=1:CurM(16,16)=1    
else
for y=1 to MatY
  for x=1 to MatX
    if rnd() =< RandL then
      CurM(x,y) = 1
    else
      CurM(x,y) = 0
    end if
  next x
  print  
next y
end if
end sub

Sub DrawM ' Draw the matrix
cursor 0,0
for y=1 to MatY
  for x=1 to MatX
    if CurM(x,y) then print A$; else print " ";
  next x
  print
next y
end sub

Sub NextGen ' Breed the next generation into a new matrix
  for y=1 to MatY
    for x=1 to MatX
      ' Find the number of live adjacent cells
      adj=CurM(x-1,y-1)+CurM(x,y-1)+CurM(x+1,y-1)+CurM(x-1,y)
      adj=adj+CurM(x+1,y)+CurM(x-1,y+1)+CurM(x,y+1)+CurM(x+1,y+1)
      CC=CurM(x,y) ' Current cell
      NC=0 ' Assume NewCell is dead
      if CC=0 then
        if adj=3 then NC=1 ' It's alive!
      else
        if adj=3 or adj=2 then NC=1 ' It's alive!
      end if
      MewM(x,y)=NC
    next x
  next y
end sub

sub UpdateM ' Update the old matrix from the new one
  for y=1 to MatY
    for x=1 to MatX
      CurM(x,y)=MewM(x,y)
    next x
  next y
end sub